@@ -32,22 +32,28 @@
 #include <sys/resource.h>
 #endif
 
-static const char usage_str[] =
-"Usage: %s [OPTION]... config.ini\n"
-"  -d, --daemon           Run in background (as a daemon)\n"
-"  -R, --restart          Do an online restart\n"
-"  -q, --quiet            Run quietly\n"
-"  -v, --verbose          Increase verbosity\n"
-"  -u, --user=<username>  Assume identity of <username>\n"
-"  -V, --version          Show version\n"
-"  -h, --help             Show this help screen and exit\n";
-
-static void usage(int err, char *exe)
+static void usage(const char *exe)
 {
-	printf(usage_str, basename(exe));
-	exit(err);
+	printf("%s is a connection pooler for PostgreSQL.\n\n", exe);
+	printf("Usage:\n");
+	printf("  %s [OPTION]... CONFIG_FILE\n", exe);
+	printf("\nOptions:\n");
+	printf("  -d, --daemon         run in background (as a daemon)\n");
+	printf("  -q, --quiet          run quietly\n");
+	printf("  -R, --reboot         do an online reboot\n");
+	printf("  -u, --user=USERNAME  assume identity of USERNAME\n");
+	printf("  -v, --verbose        increase verbosity\n");
+	printf("  -V, --version        show version, then exit\n");
+	printf("  -h, --help           show this help, then exit\n");
+	printf("\n");
+	printf("Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
+	printf("%s home page: <%s>\n", PACKAGE_NAME, PACKAGE_URL);
+	exit(0);
 }
 
+/* global libevent handle */
+struct event_base *pgb_event_base;
+
 /* async dns handler */
 struct DNSContext *adns;
 
@@ -87,6 +93,7 @@ int cf_tcp_keepalive;
 int cf_tcp_keepcnt;
 int cf_tcp_keepidle;
 int cf_tcp_keepintvl;
+int cf_tcp_user_timeout;
 
 int cf_auth_type = AUTH_MD5;
 char *cf_auth_file;
@@ -272,6 +279,7 @@ CF_ABS("tcp_keepalive", CF_INT, cf_tcp_keepalive, 0, "1"),
 CF_ABS("tcp_keepcnt", CF_INT, cf_tcp_keepcnt, 0, "0"),
 CF_ABS("tcp_keepidle", CF_INT, cf_tcp_keepidle, 0, "0"),
 CF_ABS("tcp_keepintvl", CF_INT, cf_tcp_keepintvl, 0, "0"),
+CF_ABS("tcp_user_timeout", CF_INT, cf_tcp_user_timeout, 0, "0"),
 CF_ABS("verbose", CF_INT, cf_verbose, 0, NULL),
 CF_ABS("admin_users", CF_STR, cf_admin_users, 0, ""),
 CF_ABS("stats_users", CF_STR, cf_stats_users, 0, ""),
@@ -286,7 +294,7 @@ CF_ABS("client_tls_sslmode", CF_LOOKUP(sslmode_map), cf_client_tls_sslmode, CF_N
 CF_ABS("client_tls_ca_file", CF_STR, cf_client_tls_ca_file, CF_NO_RELOAD, ""),
 CF_ABS("client_tls_cert_file", CF_STR, cf_client_tls_cert_file, CF_NO_RELOAD, ""),
 CF_ABS("client_tls_key_file", CF_STR, cf_client_tls_key_file, CF_NO_RELOAD, ""),
-CF_ABS("client_tls_protocols", CF_STR, cf_client_tls_protocols, CF_NO_RELOAD, "all"),
+CF_ABS("client_tls_protocols", CF_STR, cf_client_tls_protocols, CF_NO_RELOAD, "secure"),
 CF_ABS("client_tls_ciphers", CF_STR, cf_client_tls_ciphers, CF_NO_RELOAD, "fast"),
 CF_ABS("client_tls_dheparams", CF_STR, cf_client_tls_dheparams, CF_NO_RELOAD, "auto"),
 CF_ABS("client_tls_ecdhcurve", CF_STR, cf_client_tls_ecdhecurve, CF_NO_RELOAD, "auto"),
@@ -295,7 +303,7 @@ CF_ABS("server_tls_sslmode", CF_LOOKUP(sslmode_map), cf_server_tls_sslmode, CF_N
 CF_ABS("server_tls_ca_file", CF_STR, cf_server_tls_ca_file, CF_NO_RELOAD, ""),
 CF_ABS("server_tls_cert_file", CF_STR, cf_server_tls_cert_file, CF_NO_RELOAD, ""),
 CF_ABS("server_tls_key_file", CF_STR, cf_server_tls_key_file, CF_NO_RELOAD, ""),
-CF_ABS("server_tls_protocols", CF_STR, cf_server_tls_protocols, CF_NO_RELOAD, "all"),
+CF_ABS("server_tls_protocols", CF_STR, cf_server_tls_protocols, CF_NO_RELOAD, "secure"),
 CF_ABS("server_tls_ciphers", CF_STR, cf_server_tls_ciphers, CF_NO_RELOAD, "fast"),
 
 {NULL}
@@ -433,10 +441,11 @@ static void handle_sigterm(evutil_socket_t sock, short flags, void *arg)
 static void handle_sigint(evutil_socket_t sock, short flags, void *arg)
 {
 	log_info("got SIGINT, shutting down");
+	sd_notify(0, "STOPPING=1");
 	if (cf_reboot)
-		fatal("takeover was in progress, going down immediately");
+		die("takeover was in progress, going down immediately");
 	if (cf_pause_mode == P_SUSPEND)
-		fatal("suspend was in progress, going down immediately");
+		die("suspend was in progress, going down immediately");
 	cf_pause_mode = P_PAUSE;
 	cf_shutdown = 1;
 }
@@ -470,7 +479,7 @@ static void handle_sigusr2(int sock, short flags, void *arg)
 		cf_pause_mode = P_NONE;
 		break;
 	case P_NONE:
-		log_info("got SIGUSR1, but not paused/suspended");
+		log_info("got SIGUSR2, but not paused/suspended");
 	}
 
 	/* avoid surprise later if cf_shutdown stays set */
@@ -483,7 +492,9 @@ static void handle_sigusr2(int sock, short flags, void *arg)
 static void handle_sighup(int sock, short flags, void *arg)
 {
 	log_info("got SIGHUP, re-reading config");
+	sd_notify(0, "RELOADING=1");
 	load_config();
+	sd_notify(0, "READY=1");
 }
 #endif
 
@@ -503,30 +514,30 @@ static void signal_setup(void)
 
 	/* install handlers */
 
-	signal_set(&ev_sigusr1, SIGUSR1, handle_sigusr1, NULL);
-	err = signal_add(&ev_sigusr1, NULL);
+	evsignal_assign(&ev_sigusr1, pgb_event_base, SIGUSR1, handle_sigusr1, NULL);
+	err = evsignal_add(&ev_sigusr1, NULL);
 	if (err < 0)
-		fatal_perror("signal_add");
+		fatal_perror("evsignal_add");
 
-	signal_set(&ev_sigusr2, SIGUSR2, handle_sigusr2, NULL);
-	err = signal_add(&ev_sigusr2, NULL);
+	evsignal_assign(&ev_sigusr2, pgb_event_base, SIGUSR2, handle_sigusr2, NULL);
+	err = evsignal_add(&ev_sigusr2, NULL);
 	if (err < 0)
-		fatal_perror("signal_add");
+		fatal_perror("evsignal_add");
 
-	signal_set(&ev_sighup, SIGHUP, handle_sighup, NULL);
-	err = signal_add(&ev_sighup, NULL);
+	evsignal_assign(&ev_sighup, pgb_event_base, SIGHUP, handle_sighup, NULL);
+	err = evsignal_add(&ev_sighup, NULL);
 	if (err < 0)
-		fatal_perror("signal_add");
+		fatal_perror("evsignal_add");
 #endif
-	signal_set(&ev_sigterm, SIGTERM, handle_sigterm, NULL);
-	err = signal_add(&ev_sigterm, NULL);
+	evsignal_assign(&ev_sigterm, pgb_event_base, SIGTERM, handle_sigterm, NULL);
+	err = evsignal_add(&ev_sigterm, NULL);
 	if (err < 0)
-		fatal_perror("signal_add");
+		fatal_perror("evsignal_add");
 
-	signal_set(&ev_sigint, SIGINT, handle_sigint, NULL);
-	err = signal_add(&ev_sigint, NULL);
+	evsignal_assign(&ev_sigint, pgb_event_base, SIGINT, handle_sigint, NULL);
+	err = evsignal_add(&ev_sigint, NULL);
 	if (err < 0)
-		fatal_perror("signal_add");
+		fatal_perror("evsignal_add");
 }
 
 /*
@@ -537,7 +548,7 @@ static void go_daemon(void)
 	int pid, fd;
 
 	if (!cf_pidfile || !cf_pidfile[0])
-		fatal("daemon needs pidfile configured");
+		die("daemon needs pidfile configured");
 
 	/* don't log to stdout anymore */
 	cf_quiet = 1;
@@ -545,7 +556,7 @@ static void go_daemon(void)
 	/* send stdin, stdout, stderr to /dev/null */
 	fd = open("/dev/null", O_RDWR);
 	if (fd < 0)
-		fatal_perror("/dev/null");
+		die("could not open /dev/null: %s", strerror(errno));
 	dup2(fd, 0);
 	dup2(fd, 1);
 	dup2(fd, 2);
@@ -555,19 +566,19 @@ static void go_daemon(void)
 	/* fork new process */
 	pid = fork();
 	if (pid < 0)
-		fatal_perror("fork");
+		die("fork failed: %s", strerror(errno));
 	if (pid > 0)
 		_exit(0);
 
 	/* create new session */
 	pid = setsid();
 	if (pid < 0)
-		fatal_perror("setsid");
+		die("setsid failed: %s", strerror(errno));
 
 	/* fork again to avoid being session leader */
 	pid = fork();
 	if (pid < 0)
-		fatal_perror("fork");
+		die("fork failed: %s", strerror(errno));
 	if (pid > 0)
 		_exit(0);
 }
@@ -600,12 +611,12 @@ static void check_pidfile(void)
 	if (fd < 0) {
 		if (errno == ENOENT)
 			return;
-		fatal_perror("could not open pidfile");
+		die("could not open pidfile '%s': %s", cf_pidfile, strerror(errno));
 	}
 	res = read(fd, buf, sizeof(buf) - 1);
 	close(fd);
 	if (res <= 0)
-		fatal_perror("could not read pidfile");
+		die("could not read pidfile '%s': %s", cf_pidfile, strerror(errno));
 
 	/* parse pid */
 	buf[res] = 0;
@@ -623,11 +634,11 @@ static void check_pidfile(void)
 	log_info("stale pidfile, removing");
 	err = unlink(cf_pidfile);
 	if (err != 0)
-		fatal_perror("could not remove stale pidfile");
+		die("could not remove stale pidfile: %s", strerror(errno));
 	return;
 
 locked_pidfile:
-	fatal("pidfile exists, another instance running?");
+	die("pidfile exists, another instance running?");
 }
 
 static void write_pidfile(void)
@@ -644,10 +655,10 @@ static void write_pidfile(void)
 
 	fd = open(cf_pidfile, O_WRONLY | O_CREAT | O_EXCL, 0644);
 	if (fd < 0)
-		fatal_perror("%s", cf_pidfile);
+		die("could not open pidfile '%s': %s", cf_pidfile, strerror(errno));
 	res = safe_write(fd, buf, strlen(buf));
 	if (res < 0)
-		fatal_perror("%s", cf_pidfile);
+		die("could not write pidfile '%s': %s", cf_pidfile, strerror(errno));
 	close(fd);
 
 	/* only remove when we have it actually written */
@@ -706,7 +717,7 @@ static bool check_old_process_unix(void)
 
 	fd = socket(domain, SOCK_STREAM, 0);
 	if (fd < 0)
-		fatal_perror("cannot create socket");
+		die("could not create socket: %s", strerror(errno));
 	res = safe_connect(fd, (struct sockaddr *)&sa_un, len);
 	safe_close(fd);
 	if (res < 0)
@@ -720,7 +731,7 @@ static void main_loop_once(void)
 
 	reset_time_cache();
 
-	err = event_loop(EVLOOP_ONCE);
+	err = event_base_loop(pgb_event_base, EVLOOP_ONCE);
 	if (err < 0) {
 		if (errno != EINTR)
 			log_warning("event_loop failed: %s", strerror(errno));
@@ -738,15 +749,20 @@ static void main_loop_once(void)
 static void takeover_part1(void)
 {
 	/* use temporary libevent base */
-	void *evtmp = event_init();
+	struct event_base *evtmp;
+
+	evtmp = pgb_event_base;
+	pgb_event_base = event_base_new();
 
 	if (!cf_unix_socket_dir || !*cf_unix_socket_dir)
-		fatal("cannot reboot if unix dir not configured");
+		die("cannot reboot if unix dir not configured");
 
 	takeover_init();
 	while (cf_reboot)
 		main_loop_once();
-	event_base_free(evtmp);
+
+	event_base_free(pgb_event_base);
+	pgb_event_base = evtmp;
 }
 
 static void dns_setup(void)
@@ -755,7 +771,7 @@ static void dns_setup(void)
 		return;
 	adns = adns_create_context();
 	if (!adns)
-		fatal("dns setup failed");
+		die("dns setup failed");
 }
 
 static void xfree(char **ptr_p)
@@ -775,7 +791,7 @@ static void cleanup(void)
 	objects_cleanup();
 	sbuf_cleanup();
 
-	event_base_free(NULL);
+	event_base_free(pgb_event_base);
 
 	tls_deinit();
 	varcache_deinit();
@@ -849,6 +865,10 @@ int main(int argc, char *argv[])
 			break;
 		case 'V':
 			printf("%s\n", PACKAGE_STRING);
+			printf("libevent %s\nadns: %s\ntls: %s\n",
+			       event_get_version(),
+			       adns_get_backend(),
+			       tls_backend_version());
 			return 0;
 		case 'd':
 			cf_daemon = 1;
@@ -860,15 +880,17 @@ int main(int argc, char *argv[])
 			arg_username = optarg;
 			break;
 		case 'h':
-			usage(0, argv[0]);
+			usage(argv[0]);
 			break;
 		default:
-			usage(1, argv[0]);
+			fprintf(stderr, "Try \"%s --help\" for more information.\n", argv[0]);
+			exit(1);
 			break;
 		}
 	}
 	if (optind + 1 != argc) {
-		fprintf(stderr, "Need config file.  See pgbouncer -h for usage.\n");
+		fprintf(stderr, "%s: no configuration file specified\n", argv[0]);
+		fprintf(stderr, "Try \"%s --help\" for more information.\n", argv[0]);
 		exit(1);
 	}
 	cf_config_file = xstrdup(argv[optind]);
@@ -894,7 +916,7 @@ int main(int argc, char *argv[])
 
 	/* disallow running as root */
 	if (getuid() == 0)
-		fatal("PgBouncer should not run as root");
+		die("PgBouncer should not run as root");
 
 	admin_setup();
 
@@ -909,7 +931,7 @@ int main(int argc, char *argv[])
 		}
 	} else {
 		if (check_old_process_unix())
-			fatal("unix socket is in use, cannot continue");
+			die("unix socket is in use, cannot continue");
 		check_pidfile();
 	}
 
@@ -922,8 +944,8 @@ int main(int argc, char *argv[])
 
 	/* initialize subsystems, order important */
 	srandom(time(NULL) ^ getpid());
-	if (!event_init())
-		fatal("event_init() failed");
+	if (!(pgb_event_base = event_base_new()))
+		die("event_base_new() failed");
 	dns_setup();
 	signal_setup();
 	janitor_setup();
@@ -940,9 +962,11 @@ int main(int argc, char *argv[])
 	write_pidfile();
 
 	log_info("process up: %s, libevent %s (%s), adns: %s, tls: %s", PACKAGE_STRING,
-		 event_get_version(), event_get_method(), adns_get_backend(),
+		 event_get_version(), event_base_get_method(pgb_event_base), adns_get_backend(),
 		 tls_backend_version());
 
+	sd_notify(0, "READY=1");
+
 	/* main loop */
 	while (cf_shutdown < 2)
 		main_loop_once();
